import { RabbitSubscribe } from '@golevelup/nestjs-rabbitmq';
import { Controller } from '@nestjs/common';
import { CommandBus } from '@nestjs/cqrs';
import { UpdateUsernameCommand } from '../../commands/update-username.command';
import { Type } from '../../domain/dtos/type.enum';
import { UpdateUsernameRequest } from '../../domain/dtos/update-username.request';
import { DeleteAuthenticationRequest } from '../../domain/dtos/delete-authentication.request';
import { DeleteAuthenticationCommand } from '../../commands/delete-authentication.command';

@Controller()
export class AuthenticationMessagerController {
  constructor(private readonly _commandBus: CommandBus) {}

  @RabbitSubscribe({
    exchange: 'user',
    routingKey: 'update',
    queue: 'authentication-user-update',
  })
  public async userUpdatedHandler(message: string) {
    const updatedUser = JSON.parse(message);
    if (!updatedUser.hasOwnProperty('uuid')) throw new Error();
    if (updatedUser.hasOwnProperty('email') && updatedUser.email) {
      const updateUsernameRequest = new UpdateUsernameRequest();
      updateUsernameRequest.uuid = updatedUser.uuid;
      updateUsernameRequest.username = updatedUser.email;
      updateUsernameRequest.type = Type.EMAIL;
      await this._commandBus.execute(
        new UpdateUsernameCommand(updateUsernameRequest),
      );
    }
    if (updatedUser.hasOwnProperty('phone') && updatedUser.phone) {
      const updateUsernameRequest = new UpdateUsernameRequest();
      updateUsernameRequest.uuid = updatedUser.uuid;
      updateUsernameRequest.username = updatedUser.phone;
      updateUsernameRequest.type = Type.PHONE;
      await this._commandBus.execute(
        new UpdateUsernameCommand(updateUsernameRequest),
      );
    }
  }

  @RabbitSubscribe({
    exchange: 'user',
    routingKey: 'delete',
    queue: 'authentication-user-delete',
  })
  public async userDeletedHandler(message: string) {
    const deletedUser = JSON.parse(message);
    if (!deletedUser.hasOwnProperty('uuid')) throw new Error();
    const deleteAuthenticationRequest = new DeleteAuthenticationRequest();
    deleteAuthenticationRequest.uuid = deletedUser.uuid;
    await this._commandBus.execute(
      new DeleteAuthenticationCommand(deleteAuthenticationRequest),
    );
  }
}
