import { Module } from '@nestjs/common';
import { CqrsModule } from '@nestjs/cqrs';
import { DatabaseModule } from '../database/database.module';
import { AuthenticationController } from './adapters/primaries/authentication.controller';
import { CreateAuthenticationUseCase } from './domain/usecases/create-authentication.usecase';
import { ValidateAuthenticationUseCase } from './domain/usecases/validate-authentication.usecase';
import { AuthenticationProfile } from './mappers/authentication.profile';
import { AuthenticationRepository } from './adapters/secondaries/authentication.repository';
import { UpdateUsernameUseCase } from './domain/usecases/update-username.usecase';
import { UsernameProfile } from './mappers/username.profile';
import { AddUsernameUseCase } from './domain/usecases/add-username.usecase';
import { UpdatePasswordUseCase } from './domain/usecases/update-password.usecase';
import { DeleteUsernameUseCase } from './domain/usecases/delete-username.usecase';
import { DeleteAuthenticationUseCase } from './domain/usecases/delete-authentication.usecase';
import { RabbitMQModule } from '@golevelup/nestjs-rabbitmq';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { AuthenticationMessagerController } from './adapters/primaries/authentication-messager.controller';
import { LoggingMessager } from './adapters/secondaries/logging.messager';

@Module({
  imports: [
    DatabaseModule,
    CqrsModule,
    RabbitMQModule.forRootAsync(RabbitMQModule, {
      imports: [ConfigModule],
      inject: [ConfigService],
      useFactory: async (configService: ConfigService) => ({
        exchanges: [
          {
            name: 'user',
            type: 'topic',
          },
          {
            name: 'logging',
            type: 'topic',
          },
        ],
        uri: configService.get<string>('RMQ_URI'),
        connectionInitOptions: { wait: false },
        enableControllerDiscovery: true,
      }),
    }),
  ],
  controllers: [AuthenticationController, AuthenticationMessagerController],
  providers: [
    AuthenticationProfile,
    UsernameProfile,
    AuthenticationRepository,
    LoggingMessager,
    ValidateAuthenticationUseCase,
    CreateAuthenticationUseCase,
    AddUsernameUseCase,
    UpdateUsernameUseCase,
    UpdatePasswordUseCase,
    DeleteUsernameUseCase,
    DeleteAuthenticationUseCase,
  ],
  exports: [],
})
export class AuthenticationModule {}
