import { CommandHandler } from '@nestjs/cqrs';
import { LoggingMessager } from '../../adapters/secondaries/logging.messager';
import { UsernameRepository } from '../../adapters/secondaries/username.repository';
import { AddUsernameCommand } from '../../commands/add-username.command';
import { Username } from '../entities/username';

@CommandHandler(AddUsernameCommand)
export class AddUsernameUseCase {
  constructor(
    private readonly _usernameRepository: UsernameRepository,
    private readonly _loggingMessager: LoggingMessager,
  ) {}

  async execute(command: AddUsernameCommand): Promise<Username> {
    const { uuid, username, type } = command.addUsernameRequest;
    try {
      return await this._usernameRepository.create({
        uuid,
        type,
        username,
      });
    } catch (error) {
      this._loggingMessager.publish(
        'auth.username.add.warning',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  }
}
