import { CommandHandler } from '@nestjs/cqrs';
import { AuthenticationRepository } from '../../adapters/secondaries/authentication.repository';
import { Authentication } from '../entities/authentication';
import * as bcrypt from 'bcrypt';
import { UpdatePasswordCommand } from '../../commands/update-password.command';
import { LoggingMessager } from '../../adapters/secondaries/logging.messager';

@CommandHandler(UpdatePasswordCommand)
export class UpdatePasswordUseCase {
  constructor(
    private readonly _authenticationRepository: AuthenticationRepository,
    private readonly _loggingMessager: LoggingMessager,
  ) {}

  async execute(command: UpdatePasswordCommand): Promise<Authentication> {
    const { uuid, password } = command.updatePasswordRequest;
    const hash = await bcrypt.hash(password, 10);

    try {
      return await this._authenticationRepository.update(uuid, {
        password: hash,
      });
    } catch (error) {
      this._loggingMessager.publish(
        'auth.password.update.warning',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  }
}
