import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { Controller, UsePipes } from '@nestjs/common';
import { QueryBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { RpcValidationPipe } from 'src/utils/pipes/rpc.validation-pipe';
import { DecisionRequest } from '../../domain/dtos/decision.request';
import { Authorization } from '../../domain/entities/authorization';
import { DecisionQuery } from '../../queries/decision.query';
import { AuthorizationPresenter } from './authorization.presenter';

@UsePipes(
  new RpcValidationPipe({
    whitelist: true,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class AuthorizationController {
  constructor(
    private readonly _queryBus: QueryBus,
    @InjectMapper() private readonly _mapper: Mapper,
  ) {}

  @GrpcMethod('AuthorizationService', 'Decide')
  async decide(data: DecisionRequest): Promise<AuthorizationPresenter> {
    try {
      const authorization: Authorization = await this._queryBus.execute(
        new DecisionQuery(data.uuid, data.domain, data.action, data.context),
      );
      return this._mapper.map(
        authorization,
        Authorization,
        AuthorizationPresenter,
      );
    } catch (e) {
      throw new RpcException({
        code: 7,
        message: 'Permission denied',
      });
    }
  }
}
