import { HttpService } from '@nestjs/axios';
import { ConfigService } from '@nestjs/config';
import { Test, TestingModule } from '@nestjs/testing';
import { of } from 'rxjs';
import { OpaDecisionMaker } from '../../adapters/secondaries/opa.decision-maker';
import { Action } from '../../domain/dtos/action.enum';
import { Domain } from '../../domain/dtos/domain.enum';

const mockHttpService = {
  post: jest
    .fn()
    .mockImplementationOnce(() => {
      return of({
        status: 200,
        data: {
          decision_id: '96d99d44-e0a6-458e-a656-de2a400d60a8',
          result: {
            allow: true,
          },
        },
      });
    })
    .mockImplementationOnce(() => {
      return of({
        status: 200,
        data: {
          decision_id: '96d99d44-e0a6-458e-a656-de2a400d60a9',
          result: {
            allow: false,
          },
        },
      });
    })
    .mockImplementationOnce(() => {
      throw new Error();
    }),
};

const mockConfigService = {
  get: jest.fn().mockResolvedValue({
    OPA_URL: 'http://url/',
  }),
};

describe('OpaDecisionMaker', () => {
  let opaDecisionMaker: OpaDecisionMaker;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [],
      providers: [
        {
          provide: HttpService,
          useValue: mockHttpService,
        },
        {
          provide: ConfigService,
          useValue: mockConfigService,
        },
        OpaDecisionMaker,
      ],
    }).compile();

    opaDecisionMaker = module.get<OpaDecisionMaker>(OpaDecisionMaker);
  });

  it('should be defined', () => {
    expect(opaDecisionMaker).toBeDefined();
  });

  describe('execute', () => {
    it('should return a truthy authorization', async () => {
      const authorization = await opaDecisionMaker.decide(
        'bb281075-1b98-4456-89d6-c643d3044a91',
        Domain.USER,
        Action.READ,
        [],
      );
      expect(authorization.allow).toBeTruthy();
    });
    it('should return a falsy authorization', async () => {
      const authorization = await opaDecisionMaker.decide(
        'bb281075-1b98-4456-89d6-c643d3044a91',
        Domain.USER,
        Action.READ,
        [],
      );
      expect(authorization.allow).toBeFalsy();
    });
    it('should return a falsy authorization when an error happens', async () => {
      const authorization = await opaDecisionMaker.decide(
        'bb281075-1b98-4456-89d6-c643d3044a91',
        Domain.USER,
        Action.READ,
        [],
      );
      expect(authorization.allow).toBeFalsy();
    });
  });
});
